package stx.libjava.tools.compiler.ecj.tests;

import static org.junit.Assert.*;

import java.lang.reflect.InvocationTargetException;

import org.eclipse.jdt.internal.compiler.ClassFile;
import org.junit.Test;

import stx.libjava.tools.compiler.ecj.CompilerAdapter;

public class CompilerTests {
	
	protected class ClassLoader extends java.lang.ClassLoader {
		
		@SuppressWarnings("deprecation")
		public Class<?> load(byte[] classfile) {
			return this.defineClass(classfile, 0, classfile.length);
		}
	}

	@Test
	public void test_01() {
		ClassLoader l = new ClassLoader();
		CompilerAdapter c = new CompilerAdapter(l);
		
				
		c.compile("test.pkg.Foo", "package test.pkg; public class Foo {}");
		
		assertFalse(c.getResult().hasErrors());
		
		ClassFile[] classfiles = c.getResult().getClassFiles();
		
		assertEquals(1, classfiles.length);
		
		Class<?> clazz = l.load(classfiles[0].getBytes());
		
		assertEquals("test.pkg.Foo" , clazz.getName());
	}
	
	
	/**
	 * Tests compilation of an incomplete class
	 */
	@Test
	public void test_02() {
		ClassLoader l = new ClassLoader();
		CompilerAdapter c = new CompilerAdapter(l);
		
				
		c.compile("test.pkg.Foo", "package test.pkg; public class Foo { public void f() { g(); } }");		
		ClassFile[] classfiles = c.getResult().getClassFiles();
		
		assertEquals(1, classfiles.length);
		
		Class<?> fooClass = l.load(classfiles[0].getBytes());
		
		assertEquals("test.pkg.Foo" , fooClass.getName());
		assertEquals(1, fooClass.getDeclaredMethods().length);
		
		try {
			fooClass.getMethod("f").invoke(fooClass.newInstance());
			assertTrue(false);		
		} catch (InvocationTargetException ite) {
			assertTrue(ite.getCause() instanceof Error);
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			assertTrue(false);		
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			assertTrue(false);		
		} catch (NoSuchMethodException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			assertTrue(false);		
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			assertTrue(false);		
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			assertTrue(false);		
		}
	}

	/**
	 * Tests compilation of depending classes
	 */
	@Test
	public void test_03() {
		ClassLoader l = new ClassLoader();
		CompilerAdapter c = new CompilerAdapter(l);
				
		c.compile("test.pkg.Foo", "package test.pkg; public class Foo { public void f() {  } }");
		assertEquals(1, c.getClassFiles().length);
		assertFalse(c.getResult().hasErrors());
		
		c.compile("test.pkg.Bar", "package test.pkg; public class Bar { public void f() { new Foo(); } }");
		assertEquals(1, c.getClassFiles().length);
		assertFalse(c.getResult().hasErrors());
		
	}

	@Test
	public void test_04() {
		ClassLoader l = new ClassLoader();
		CompilerAdapter c = new CompilerAdapter(l);
				
		c.compile("test.pkg.Foo", "package test.pkg; import static java.lang.System.*; public class Foo {}");
		
		assertFalse(c.getResult().hasErrors());
		
		ClassFile[] classfiles = c.getResult().getClassFiles();
		
		assertEquals(1, classfiles.length);
		
		Class<?> clazz = l.load(classfiles[0].getBytes());
		
		assertEquals("test.pkg.Foo" , clazz.getName());
	}
	
	/**
	 * Tests compilation of depending classes
	 */
	@Test
	public void test_03b() {
		ClassLoader l = new ClassLoader();
		CompilerAdapter c1 = new CompilerAdapter(l);
				
		c1.compile("test.pkg.Foo", "package test.pkg; public class Foo { public void f() {  } }");
		assertEquals(1, c1.getClassFiles().length);
		assertFalse(c1.getResult().hasErrors());
		l.load(c1.getResult().getClassFiles()[0].getBytes());
		
		CompilerAdapter c2 = new CompilerAdapter(l);
		c2.compile("test.pkg.Bar", "package test.pkg; public class Bar { public void f() { new Foo(); } }");
		assertEquals(1, c2.getClassFiles().length);
		assertFalse(c2.getResult().hasErrors());
		
	}

	@Test
	public void test_05() {
		ClassLoader l = new ClassLoader();
		CompilerAdapter c1 = new CompilerAdapter(l);
				
		c1.compile("Foo", "public class Foo { public int f() { return");
		assertEquals(1, c1.getClassFiles().length);
		assertTrue(c1.getResult().hasSyntaxError);
	}
	
}
